<?php
if (!defined('_PS_VERSION_'))
    exit;

class MojeId extends Module
{
    // Used to build the required output for display Configuration form in the Back Office.
    private $_html = '';

    // Used to store error during validation followed by display it to user.
    private $_postErrors = array();

    public function __construct()
    {

        $this->name = 'mojeid';
        $this->tab = 'front_office_features';
        $this->version = '1.1';
        $this->author = 'Dominik Ulrich (www.openservis.cz)';
        $this->need_instance = 0;
        $this->is_configurable = 1;
        $this->ps_versions_compliancy = array('min' => '1.4', 'max' => '1.4');
        $this->credits = 'Tento modul vytvořil Dominik Shaim v rámci služby <a href="http://openservis.cz/?from=' . urlencode($this->name) . '&web=' . urlencode($_SERVER['HTTP_HOST']) . '" title="www.openservis.cz" target="_blank">www.openservis.cz</a>';

        parent::__construct();

        $this->displayName = $this->l('mojeID login support');
        $this->description = $this->l('Log in user from mojeID account.');

        $this->beforeUninstall = $this->l('This will delete all data about users connected with mojeID. Do you want to continue?');
        if (version_compare(_PS_VERSION_, '1.4', '<') || version_compare(_PS_VERSION_, '1.5', '>=')) {
            $this->warning = $this->l('Tento modul je určen pouze pro Prestashop 1.4.');
        }
    }

    public function install()
    {
        global $cookie;
        if (version_compare(_PS_VERSION_, '1.4', '<') || version_compare(_PS_VERSION_, '1.5', '>=')) {
            return false;
        }
        $cms = new CMS();
        $cms->meta_title[$cookie->id_lang] = $this->l('Výhody mojeID');
        $cms->link_rewrite[$cookie->id_lang] = $this->l('vyhody-mojeid');
        $cms->id_cms_category = 1;
        $cms->active = 1;
        $cmscontent = file_get_contents(_PS_MODULE_DIR_ . $this->name . '/cms/cms_template.html');
        // /testshop/
        // http://localhost/testshop/modules/mojeid/mojeid_register.php
        $cmscontent = str_replace('{BASE_URI}', __PS_BASE_URI__, $cmscontent);
        $cmscontent = str_replace('{REGISTER_LINK}', __PS_BASE_URI__ . 'modules/mojeid/mojeid_register.php', $cmscontent);
        $cms->content[$cookie->id_lang] = $cmscontent;
        $cms->save();

        if (!parent::install()
            OR !$this->registerHook('header')
            OR !$this->registerHook('myAccountBlock')
            OR !$this->registerHook('adminOrder')
            OR !$this->registerHook('LeftColumn')
            OR !$this->registerHook('RightColumn')
            OR !Configuration::updateValue($this->name . '_poweredimage', 'A1')
            OR !Configuration::updateValue($this->name . '_loginimage', 'black/155x24')
            OR !Configuration::updateValue($this->name . '_cmsid', (int)$cms->id)
            OR !Configuration::updateValue($this->name . '_showlogo', 1)
        )
            return false;

        if (!$this->_setUrlInXmlFile()) {

            return false;
        }
        $sql = 'ALTER TABLE `' . _DB_PREFIX_ . 'customer` ADD `mojeid` VARCHAR( 100 ) NULL DEFAULT NULL ,
						ADD `auth_level` VARCHAR( 100 ) NULL DEFAULT NULL ,
						ADD `mojeid_origin` VARCHAR( 3 ) NOT NULL DEFAULT \'Ne\' ';
        if (!Db::getInstance()->Execute($sql)) {

            return false;
        }
        return true;
    }

    public function uninstall()
    {


        $cms = new CMS((int)Configuration::get($this->name . '_cmsid'));

        if (
            parent::uninstall() == false
            OR $this->unregisterHook('header') == false
            OR $this->unregisterHook('myAccountBlock') == false
            OR $this->unregisterHook('adminOrder') == false
            OR $this->unregisterHook('LeftColumn') == false
            OR $this->unregisterHook('RightColumn') == false
            OR Configuration::deleteByName($this->name . '_poweredimage') == false
            OR Configuration::deleteByName($this->name . '_loginimage') == false
            OR Configuration::deleteByName($this->name . '_cmsid') == false
            OR Configuration::deleteByName($this->name . '_showlogo') == false
            OR $cms->delete() == false
        )
            return false;


        $sql = 'ALTER TABLE `' . _DB_PREFIX_ . 'customer`
                              DROP `mojeid`,
                              DROP `auth_level`,
                              DROP `mojeid_origin`';

        if (!Db::getInstance()->Execute($sql)) {
            return false;
        }


        return true;
    }

    /**
     ** CONFIGURATION INTERFACE
     */

    /*
     ** Display settings form content
    */
    public function getContent()
    {

        if (Tools::isSubmit('submit')) {
            $this->_postValidation();
            if (sizeof($this->_postErrors) == 0) {
                $this->_postProcess();
            }

            if (sizeof($this->_postErrors) != 0) {
                foreach ($this->_postErrors AS $err) {
                    $this->_html .= '<div class="alert error">' . $err . '</div>';
                }
            }
        }
        $this->_displayForm();
        return $this->_html;
    }

    /**
     ** Validate configuration form values
     */
    private function _postValidation()
    {
        if (!Validate::isString(Tools::getValue('poweredimage'))) {
            $this->_postErrors[] = $this->l('Image type') . ' ' . $this->l('is not valid');
        }
        if (!Validate::isString(Tools::getValue('loginimage'))) {
            $this->_postErrors[] = $this->l('Image type') . ' ' . $this->l('is not valid');
        }
        if (!Validate::isBool(Tools::getValue('showlogo'))) {
            $this->_postErrors[] = $this->l('Show logo') . ' ' . $this->l('is not valid');
        }
    }

    /**
     ** Function is invoked when no validation errors are present
     */
    private function _postProcess()
    {
        Configuration::updateValue($this->name . '_poweredimage', Tools::getValue('poweredimage'));
        Configuration::updateValue($this->name . '_loginimage', Tools::getValue('loginimage'));
        Configuration::updateValue($this->name . '_showlogo', intval(Tools::getValue('showlogo')));

        $this->_html .= '<div class="conf confirm">' . $this->l('Settings updated') . '</div>';
    }

    private function _displayForm()
    {
        global $cookie;
        $this->_html .= '<form action="' . Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']) . '" method="post">';
        $this->_html .= '<fieldset>';
        $this->_html .= '<legend>' . $this->l('Settings') . '</legend>';

        $poweredImage = Configuration::get($this->name . '_poweredimage');
        $poweredImages = Array('A1', 'A2', 'A3', 'A4', 'B1', 'B2', 'B3', 'B4');
        $lang = $iso = Language::getIsoById($cookie->id_lang);
        $iter = 0;

        $this->_html .= '<label>' . $this->l('Choose an image') . '</label>';
        $this->_html .= '<div class="margin-form clearfix">';
        $this->_html .= '<table><tr style="text-align: center">';
        foreach ($poweredImages as $img) {
            $this->_html .= '<td>';
            $this->_html .= '
    		<label class="t" for="' . $img . '">
    			<img style="padding: 5px 0; margin: 0 auto; display: block;" src="' . $this->_path . 'img/small/' . $lang . '/' . $img . '.png" alt="' . $img . '" title="' . $img . '" />
    		</label>';
            $this->_html .= '<input type="radio" name="poweredimage" id="' . $img . '" value="' . $img . '" ' . ($img == $poweredImage ? 'checked="checked" ' : '') . '/>';
            $this->_html .= '</td>';
            if (++$iter == 4)
                $this->_html .= '</tr><tr style="text-align: center">';
        }
        $this->_html .= '</tr></table>';
        $this->_html .= '</div>';

        $this->_html .= '<label>' . $this->l('Show "Powered by mojeID"') . '</label>';
        $this->_html .= '<div class="margin-form">';
        $this->_html .= '<input type="radio" name="showlogo" id="showlogo_on" value="1" ' . (Tools::getValue('showlogo', Configuration::get($this->name . '_showlogo')) ? 'checked="checked" ' : '') . '/>';
        $this->_html .= '<label class="t" for="showlogo_on"> <img src="' . _PS_IMG_ . 'admin/enabled.gif" alt="' . $this->l('Enabled') . '" title="' . $this->l('Enabled') . '" /></label>';
        $this->_html .= '<input type="radio" name="showlogo" id="showlogo_off" value="0" ' . (!Tools::getValue('showlogo', Configuration::get($this->name . '_showlogo')) ? 'checked="checked" ' : '') . '/>';
        $this->_html .= '<label class="t" for="showlogo_off"> <img src="' . _PS_IMG_ . 'admin/disabled.gif" alt="' . $this->l('Disabled') . '" title="' . $this->l('Disabled') . '" /></label>';
        $this->_html .= '<p class="clear">' . $this->l('Check if you want to show logo') . '</p>';
        $this->_html .= '</div>';

        $loginImage = Configuration::get($this->name . '_loginimage');
        $loginImages = Array('silver/16x16', 'silver/32x32', 'silver/92x32', 'silver/155x24', 'silver/162x32',
            'black/16x16', 'black/32x32', 'black/92x32', 'black/155x24', 'black/162x32');

        $this->_html .= '<label>' . $this->l('Choose an image') . '</label>';
        $this->_html .= '<div class="margin-form clearfix" style="width: 580px">';
        foreach ($loginImages as $img) {
            $this->_html .= '<input type="radio" name="loginimage" id="' . $img . '" value="' . $img . '" ' . ($img == $loginImage ? 'checked="checked" ' : '') . '/>';
            $this->_html .= '
    		<label class="t" for="' . $img . '">
    			<img style="padding-top: 10px;" src="' . $this->_path . 'img/' . $img . '.png" alt="' . $img . '" title="' . $img . '" />
    		</label>';
        }
        $this->_html .= '</div>';

        $this->_html .= '<input type="submit" name="submit" value="' . $this->l('Update') . '" class="button" />';
        $this->_html .= '</fieldset>';
        $this->_html .= '</form>';
        $this->_html .= "<br /><br /><div><small>{$this->credits}</small></div>";
    }

    public function hookHeader($params)
    {
        return $this->display(__FILE__, 'meta.tpl');
    }

    public function hookMyAccountBlock($params)
    {
        global $smarty, $link;
        $customer = new Customer($params['cookie']->id_customer);
        if ($customer->auth_level) {
            $smarty->assign('logged', 1);
            $smarty->assign('authLevel', $customer->auth_level);
        } else {
            $cmsPage = new CMS(Configuration::get($this->name . '_cmsid'));
            $smarty->assign('logged', 0);
            $smarty->assign('loginImage', Configuration::get('mojeid_loginimage'));
            $smarty->assign('cmsLink', $link->getCMSLink($cmsPage->id, $cmsPage->link_rewrite[$params['cookie']->id_lang]));
            $smarty->assign('registerLink', __PS_BASE_URI__ . 'modules/mojeid/mojeid_register.php');
        }
        return $this->display(__FILE__, 'my-account.tpl');
    }

    public function hookLeftColumn($params)
    {
        global $smarty, $link;

        if (Configuration::get($this->name . '_showlogo') && !isset($this->added)) {
            $this->added = true;
            $iso = Language::getIsoById($params['cookie']->id_lang);
            $cmsPage = new CMS(Configuration::get($this->name . '_cmsid'));
            $cmslink = $link->getCMSLink($cmsPage->id, $cmsPage->link_rewrite[$params['cookie']->id_lang]);

            $smarty->assign('img', $this->_path . 'img/small/' . $iso . '/' . Configuration::get($this->name . '_poweredimage') . '.png');
            $smarty->assign('cmslink', $cmslink);
            return $this->display(__FILE__, 'leftColumn.tpl');
        }
    }


    public function hookRightColumn($params)
    {
        return $this->hookLeftColumn($params);
    }

    public function hookHome($params)
    {
        return $this->hookLeftColumn($params);
    }

    public function hookAdminOrder($params)
    {
        $order = new Order($params['id_order']);
        $customer = new Customer($order->id_customer);
        return '
    	<br /><fieldset style="width: 400px;">
    	<legend><img src="' . $this->_path . 'logo.gif">mojeID</legend>
    	<p>
    		status: <strong>' . $customer->auth_level . '</strong>
    	</p>
    	</fieldset>';
    }

    private function _setUrlInXmlFile()
    {
        $modulePath = _PS_MODULE_DIR_ . $this->name . '/';
        $xmlFilename = 'xrds.xml';
        $protocol = 'http://';
        $protocol_ssl = 'https://';
        $protocol_link = (Configuration::get('PS_SSL_ENABLED')) ? $protocol_ssl : $protocol;
        $server_host = htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8');
        $xml = '<?xml version="1.0" encoding="UTF-8"?>
			    	<xrds:XRDS xmlns:xrds="xri://$xrds" xmlns="xri://$xrd*($v*2.0)">
			    	<XRD><Service><Type>http://specs.openid.net/auth/2.0/return_to</Type>
			    	<URI>' . $protocol_link . $server_host . __PS_BASE_URI__ . 'authentication.php</URI>
			    	</Service>
			    	</XRD>
			    	</xrds:XRDS>';
        if (!is_writable($modulePath)) {
            return false;
        }
        if (!is_writable($modulePath . $xmlFilename)) {
            return false;
        }
        $fileHandle = fopen($modulePath . $xmlFilename, 'w');
        if ($fileHandle == false) {
            return false;
        }
        if (!fwrite($fileHandle, $xml)) {
            return false;
        }
        fclose($fileHandle);
        return true;
    }

    public static function getCMSId()
    {
        return Configuration::get('mojeid_cmsid');
    }

    public static function getLoginImage()
    {
        return Configuration::get('mojeid_loginimage');
    }
}
